# -*- coding: utf-8 -*-
"""
General caching functionality. This module defines:

1. the Cache class
2. generic cache functions to store daily results

Created on Thu Jan  7 09:34:04 2016

@author: roel
"""
import os
import numpy as np
import pandas as pd
import dateutil
from opengrid import config
cfg = config.Config()
from opengrid.library import misc
from opengrid.library import analysis
from tqdm import tqdm
import pickle

class Cache(object):
    """
    A class to handle daily aggregated data or intermediate results
    
    The file format for the data is result_sensor.csv
    """
    
    def __init__(self, variable, folder=None):
        """
        Create a cache object specifically for the specified variable
        
        Arguments
        ---------
        variable : str
            The name of the aggregated variable to be stored.
            Examples: elec_standby, water_max, ...
        folder : path
            Path where the files are stored
            If None, use the path specified in the opengrid configuration
            
        """
        self.variable = variable
        if folder is None:
            try:
                self.folder = os.path.join(os.path.abspath(cfg.get('data', 'folder')),'cache_day')
            except:
                raise ValueError("Specify a folder, either in the opengrid.cfg or when creating this cache object.")
        else:
            self.folder = os.path.abspath(folder)
            
        if not os.path.exists(self.folder):
            print("This folder does not exist: {}, it will be created".format(self.folder))
            os.mkdir(self.folder)
            
        print("Cache object created for variable: {}".format(self.variable))

   
    def _load(self, sensorkey):
        """
        Return a dataframe with cached data for this sensor.  If there is no
        cached data, return an empty dataframe

        Arguments
        ---------
        sensor : str
            sensor key, unique string identifier of the sensor.

        Returns
        -------
        df : tz-aware dataframe with cached daily results or empty dataframe.
        
        """
        # Find the file and read into a dataframe
        filename = self.variable + '_' + sensorkey + '.pkl'
        path = os.path.join(self.folder, filename)
        
        if not os.path.exists(path):
            #print("Could not find {}".format(path))
            return pd.DataFrame()
        
        df = pickle.load(open(path, "rb"))
        if isinstance(df, pd.Series):
            df = pd.DataFrame(df)
        return df
    
    
    def _write_single(self, df):
        """
        Write the dataframe with single sensor to csv according to the filename conventions

        Arguments
        ---------
        df : pandas DataFrame or Series
        """

        # handle both cases correctly: dataframe or series
        if isinstance(df, pd.DataFrame):
            if len(df.columns) != 1:
                raise ValueError("Wrong number of columns")
            df_temp = df.copy()
        elif isinstance(df, pd.Series):
            if df.name is None:
                raise ValueError("pandas Series needs a name with sensor id")
            df_temp = pd.DataFrame(df)

        df_temp = df_temp.dropna()

        sensor = df_temp.columns[0]
        # Find the file and read into a dataframe
        filename = self.variable + '_' + sensor + '.pkl'
        path = os.path.join(self.folder, filename)

        pickle.dump(df_temp, open(path, "wb"))

        return True

    def _write(self, df):
        """
        Write the dataframe to csv according to the filename conventions

        Parameters
        ----------
        df : pandas DataFrame or Series
            This can be a multiple-column dataframe.  Each columns is a sensor_id.

        Returns
        -------
        True if all data is cached successfully

        Notes
        -----
        For each sensor found in df, a csv is saved to disk with filename
        self.folder/result_sensor.csv
        """

        if isinstance(df, pd.Series):
            return self._write_single(df)
        else:
            for sensor in df.columns:
                self._write_single(df[sensor])
            return True
    
    def get(self, sensors, start=None, end=None):
        """
        Return a dataframe with cached data for these sensors
        
        Arguments
        ---------
        sensors : list with Sensor objects
            Each element is a Sensor object with attribute 'key' as sensor_id
        start, end : Datetime, float, int, string or pandas Timestamp
            Anything that can be parsed into a pandas.Timestamp
            If None, return all cached data available
            
        Returns
        -------
        df : pandas DataFrame
            With single column, daily datetimeindex 
            and curtailed if start and/or end are given.
        """

        if not isinstance(sensors, list):
            raise TypeError("Sensors has to be a list with Sensor objects, not a {}".format(type(sensors)))

        dfs = []
        for sensor in sensors:
            df = self._load(sensor.key)
            if not df.empty:
                dfs.append(df)
        if dfs:
            df = pd.concat(dfs, axis=1)
            try:
                df.index = df.index.tz_convert('Europe/Brussels')
            except TypeError:
                df.index = df.index.tz_localize('Europe/Brussels')
        else:
            print("No cached sensordata found.")
            df = pd.DataFrame()

        if len(df) == 0:
            return df
        
        if (start is None) & (end is None):
            # no truncating
            return df
        
        # truncate
        if start is None:
            t_start = misc.parse_date(0)
        else:
            t_start = misc.parse_date(start)
        
        if end is None:
            t_end = misc.parse_date('21000101')
        else:
            t_end = misc.parse_date(end)
            
        return df.loc[t_start:t_end,:]
   
    
    def check_df(self, df):
        """
        Verify that the dataframe is acceptable as a daily aggregation variable
        
        Arguments
        ---------
        df : pandas DataFrame
            
        Returns
        --------
        True/False
        
        Return False when the dataframe is empty or when the index does not have a daily frequency

        """
        if len(df) == 0:
            print("Empty dataframe")
            return False
        
        if df.index.freqstr != 'D':
            if df.index.freqstr is not None:
                print("Wrong frequency of the index: '{}' (instead of 'D')".format(df.index.freqstr))
                return False
            else:
                # The df.index does not have a freqstr attribute for some reason. 
                # verify the frequency manually
                interval = np.round((df.index[-1] - df.index[0]).total_seconds()/(len(df.index)-1))
                if not interval == 86400:
                    print("Wrong frequency of the index: mean interval = {}s (instead of 86400)".format(interval))
                    return False
                
        return True
    

    def _update_single(self, df):
        """
        For a single sensor: update the stored dataframe with the given one

        Arguments
        ---------
        df : pandas DataFrame or Series
            If a dataframe, can only contain a single column with the sensor_id
            If a Series, the name attribute is the sensor_id

        Returns
        --------
        True if successfully updated
        """
        
        if not self.check_df(df):
            raise ValueError("Dataframe or Series not acceptable for updating.")

        # handle both cases correctly: dataframe or series
        if isinstance(df, pd.DataFrame):
            if len(df.columns) != 1:
                raise ValueError("Wrong number of columns")
            df_temp = df.copy()
        elif isinstance(df, pd.Series):
            if df.name is None:
                raise ValueError("pandas Series needs a name with sensor id")
            df_temp = pd.DataFrame(df)

        # Find the file and read into a dataframe
        sensor = df_temp.columns[0]
        df_old = self._load(sensor) #
        df_old.update(df_temp)
        df_res = df_old.combine_first(df_temp)
        self._write(df_res)
        return True


    def update(self, df):
        """
        Updates the cached data for each sensor in df

        Arguments
        ----------
        df : pandas DataFrame or Series
            This can be a multiple-column dataframe.  Each columns is a sensor_id.

        Returns
        -------
        True if all data is cached successfully

        Notes
        -----
        For each sensor found in df, a csv is saved to disk with filename
        self.folder/result_sensor.csv. If such a file was already present,
        the values are updated with the ones provided in df (will overwrite
        overlapping days).
        """

        if isinstance(df, pd.Series):
            return self._update_single(df)
        else:
            for sensor in df.columns:
                try:
                    self._update_single(df[sensor])
                except ValueError:
                    pass
            return True


def cache_results(hp, sensors, resultname, AnalysisClass, chunk=True, **kwargs):
    """
    Run an analysis on a set of sensors and cache the results

    Parameters
    ----------
    sensors : list with Sensor objects
        Each element is a Sensor object with attribute 'key' as sensor_id
    AnalysisClass : object
        Class from the analysis library for the doing the analysis
    resultname : string
        Name of the cached variable, eg. elect_standby or water_daily_max
    kwargs : dict
        Additional keyword arguments are passed to the instantiation of the analysis class
    chunk : boolean, default=True
        If True, cache day_by_day to reduce memory use.

    Returns
    -------
    True if all data has been successfully cached.

    """

    # The method would run perfectly on all sensorids at once.
    # However, this leads to large dataframes and large RAM use.
    # Therefore, we create a for loop over the sensor ids
    # update: to reduce RAM use, we add another loop to run over the days

    import importlib
    cache = Cache(variable=resultname)

    for sensor in tqdm(sensors):
        # Get whatever is available as cache
        # and only extract timeseries from tmpos since the last day
        df_cached = cache.get([sensor])
        try:
            last_day = df_cached.index[-1]
        except IndexError:
            last_day = pd.Timestamp('2013-01-01', tz='UTC')

        if chunk:
            for d in pd.DatetimeIndex(start=last_day, freq='D', end=pd.Timestamp.today()):
                # get new data for a single day, full resolution
                df_new = hp.get_data(sensors = [sensor], head=d, tail=d + pd.Timedelta(days=1))

                # apply the method
                df_day = AnalysisClass(df_new, **kwargs).result

                # cache the results
                cache.update(df_day)
        else:
            # get new data, full resolution
            df_new = hp.get_data(sensors=[sensor], head=last_day)

            # apply the method
            df_day = AnalysisClass(df_new, **kwargs).result

            # cache the results
            cache.update(df_day)
    return True








# -*- coding: utf-8 -*-
"""
Created on Thu Jan  7 09:36:16 2016

@author: roel
"""

# -*- coding: utf-8 -*-
"""
Created on Mon Dec 30 02:37:25 2013

@author: roel
"""

import os, sys
import unittest
import inspect
import numpy as np
import pdb
import pandas as pd
import pytz

test_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
os.chdir(test_dir)
# add the path to opengrid to sys.path
sys.path.insert(1, os.path.join(test_dir, os.pardir, os.pardir, os.pardir))
from opengrid.library import caching
from opengrid.library.houseprint import Sensor

# Note: there is a opengrid.cfg in the test_dir which is loaded here!!
from opengrid import config
cfg = config.Config()

class CacheTest(unittest.TestCase):
    
    def test_init(self):
        """Check if correct folder is used"""
        
        ch = caching.Cache('standby', folder=os.getcwd())
        self.assertEqual(ch.folder, os.getcwd())
        
        ch = caching.Cache('water_standby')
        self.assertEqual(ch.folder, os.path.join(os.getcwd(), 'data', 'cache_day'))
        
        
    def test_load(self):
        """Load and parse a cached object correctly"""
        ch = caching.Cache('elec_standby')
        df = ch._load('mysensor')
        
        self.assertTrue((df.index == pd.DatetimeIndex(start='20160101', freq='D', periods=365, tz='UTC')).all())
        self.assertEqual(df.columns, ['mysensor'])
        
        
    def test_get_raises(self):
        """Raise TypeError when argument sensors is not a list"""
        ch = caching.Cache('elec_standby')
        mysensor = Sensor(key='mysensor', device=None, site='None', type=None, description=None,system=None,
                                quantity=None,unit=None,direction=None,tariff=None,cumulative=None)
        self.assertRaises(TypeError, ch.get, mysensor)

    def test_get_single(self):
        """Obtain cached results and return a correct dataframe"""
        ch = caching.Cache('elec_standby')
        mysensor = Sensor(key='mysensor', device=None, site='None', type=None, description=None,system=None,
                                quantity=None,unit=None,direction=None,tariff=None,cumulative=None)
        df = ch.get([mysensor])
        
        self.assertTrue((df.index == pd.DatetimeIndex(start='20160101', freq='D', periods=365, tz='Europe/Brussels')).all())
        self.assertEqual(df.columns, ['mysensor'])
        
        df = ch.get([mysensor], end='20160115')
        self.assertTrue((df.index == pd.DatetimeIndex(start='20160101', freq='D', periods=15, tz='Europe/Brussels')).all())
        
        df = ch.get([mysensor], start = '20160707', end='20160708')
        self.assertTrue((df.index == pd.DatetimeIndex(start='20160707', freq='D', periods=2, tz='Europe/Brussels')).all())
        # self.assertFalse(df.index.tz is None, "Returned dataframe is tz-naive") #Removed this test on 21/04/16 after pull request #136 and discussion in #135.
        # However, we need to finalise the discussion: a date is also timezone dependent...


                         
    def test_get_multiple(self):
        """Obtain cached results and return a correct dataframe"""
        ch = caching.Cache('elec_standby')
        mysensor = Sensor(key='mysensor', device=None, site='None', type=None, description=None,system=None,
                                quantity=None,unit=None,direction=None,tariff=None,cumulative=None)
        mysensor2 = Sensor(key='mysensor2', device=None, site='None', type=None, description=None,system=None,
                                quantity=None,unit=None,direction=None,tariff=None,cumulative=None)
        df = ch.get([mysensor, mysensor2], end='20160104')

        self.assertTrue((df.index == pd.DatetimeIndex(start='20160101', freq='D', periods=4, tz='Europe/Brussels')).all())
        self.assertListEqual(df.columns.tolist(), ['mysensor', 'mysensor2'])
        self.assertEqual(df.ix[1, 'mysensor2'], 5)
        self.assertTrue(np.isnan(df.ix[3, 'mysensor2']))


    def test_check_df_series(self):
        """check if series is not empty and has daily frequency"""
        ch = caching.Cache('elec_standby')

        df = pd.Series()
        self.assertFalse(ch.check_df(df))

        index = pd.DatetimeIndex(start='20160101', freq='D', periods=3, tz='Europe/Brussels')
        ts = pd.Series(index=index, data=np.random.randn(3), name='A')
        self.assertTrue(ch.check_df(ts))

    def test_check_df(self):
        """check if dataframe is not empty and has daily frequency"""
        ch = caching.Cache('elec_standby')

        df = pd.DataFrame()
        self.assertFalse(ch.check_df(df))

        index = pd.DatetimeIndex(start='20160101', freq='D', periods=3, tz='Europe/Brussels')
        df = pd.DataFrame(index=index, data=np.random.randn(3,2), columns=['A', 'B'])
        self.assertTrue(ch.check_df(df))
        
        index = pd.DatetimeIndex(['20160201', '20160202', '20160203'], tz='Europe/Brussels')
        df = pd.DataFrame(index=index, data=np.random.randn(3), columns=['A'])
        self.assertTrue(ch.check_df(df))
        
        index = pd.DatetimeIndex(['20160201', '20160202', '20160204'], tz='Europe/Brussels')
        df = pd.DataFrame(index=index, data=np.random.randn(3), columns=['A'])
        self.assertFalse(ch.check_df(df))

    def test_write_single1(self):
        """Write dataframe with single columns only"""
        ch = caching.Cache('elec_temp')

        # write a dataframe with single column
        index = pd.DatetimeIndex(start='20160101', freq='D', periods=3, tz='Europe/Brussels')
        df = pd.DataFrame(index=index, data=np.random.randn(3), columns=['testsensor'])
        expected_path = os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor.pkl')
        self.assertFalse(os.path.exists(expected_path))
        try:
            ch._write_single(df)
            self.assertTrue(os.path.exists(expected_path))
        except:
            raise
        finally:
            os.remove(expected_path)

        # raise ValueError on dataframe with multiple columns
        df = pd.DataFrame(index=index, data=np.random.randn(3,2), columns=['testsensor1', 'testsensor2'])
        self.assertRaises(ValueError, ch._write_single, df)
        
    def test_write_single2(self):
        """Write timeseries """
        ch = caching.Cache('elec_temp')

        # write a dataframe with single column
        index = pd.DatetimeIndex(start='20160101', freq='D', periods=3, tz='Europe/Brussels')
        df = pd.Series(index=index, data=np.random.randn(3), name='testsensor_series')
        expected_path = os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor_series.pkl')
        self.assertFalse(os.path.exists(expected_path))
        try:
            ch._write_single(df)
            self.assertTrue(os.path.exists(expected_path))
        except:
            raise
        finally:
            os.remove(expected_path)

        # raise ValueError on series without name
        df = pd.Series(index=index, data=np.random.randn(3))
        self.assertRaises(ValueError, ch._write_single, df)

    def test_write(self):
        """Write dataframe with multiple columns"""
        ch = caching.Cache('elec_temp')

        # write a dataframe with single column
        index = pd.DatetimeIndex(start='20160101', freq='D', periods=3, tz='Europe/Brussels')
        df = pd.DataFrame(index=index, data=np.random.randn(3,2), columns=['testsensor1', 'testsensor2'])
        expected_path1 = os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor1.pkl')
        expected_path2 = os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor2.pkl')
        self.assertFalse(os.path.exists(expected_path1))
        self.assertFalse(os.path.exists(expected_path2))

        try:
            ch._write(df)
            self.assertTrue(os.path.exists(expected_path1))
            self.assertTrue(os.path.exists(expected_path2))
        except:
            raise
        finally:
            os.remove(expected_path1)
            os.remove(expected_path2)

    def test_update_single(self):
        """Update an existing cached sensor with new information"""

        ch = caching.Cache('elec_temp')
        testsensor = Sensor(key='testsensor', device=None, site='None', type=None, description=None,system=None,
                                quantity=None,unit=None,direction=None,tariff=None,cumulative=None)

        try:
            # write a dataframe with single column
            index = pd.DatetimeIndex(start='20160101', freq='D', periods=3, tz='Europe/Brussels')
            df = pd.DataFrame(index=index, data=[0,1,2], columns=['testsensor'])
            expected_path = os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor.pkl')
            self.assertFalse(os.path.exists(expected_path))
            ch._write_single(df)

            index = pd.DatetimeIndex(start='20160103', freq='D', periods=3, tz='Europe/Brussels')
            df_new = pd.DataFrame(index=index, data=[100,200,300], columns=['testsensor'])
            ch.update(df_new)
            df_res = ch.get([testsensor])
            
            self.assertEqual(df_res.iloc[1,0], 1)
            self.assertEqual(df_res.iloc[2,0], 100)
            self.assertEqual(df_res.iloc[4,0], 300)
        except:
            raise
        finally:
            os.remove(os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor.pkl'))

    def test_update_multiple(self):
        """Update an existing cached sensor with new information"""

        ch = caching.Cache('elec_temp')
        testsensor2 = Sensor(key='testsensor2', device=None, site='None', type=None, description=None,system=None,
                                quantity=None,unit=None,direction=None,tariff=None,cumulative=None)

        # write a dataframe with two columns
        index = pd.DatetimeIndex(start='20160101', freq='D', periods=3, tz='Europe/Brussels')
        df = pd.DataFrame(index=index, data=dict(testsensor1=[0,1,2], testsensor2= [0,1,2]))
        expected_path1 = os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor1.pkl')
        expected_path2 = os.path.join(test_dir, cfg.get('data', 'folder'), 'cache_day', 'elec_temp_testsensor2.pkl')
        self.assertFalse(os.path.exists(expected_path1))
        self.assertFalse(os.path.exists(expected_path2))
        try:
            ch.update(df)
            self.assertTrue(os.path.exists(expected_path1))
            self.assertTrue(os.path.exists(expected_path2))

            index = pd.DatetimeIndex(start='20160103', freq='D', periods=3, tz='Europe/Brussels')
            df_new = pd.DataFrame(index=index, data=dict(testsensor1=[100,200,300], testsensor2=[100,200,300]))
            ch.update(df_new)
            df_res = ch.get([testsensor2])

            self.assertEqual(df_res.iloc[1,0], 1)
            self.assertEqual(df_res.iloc[2,0], 100)
            self.assertEqual(df_res.iloc[4,0], 300)
        except:
            raise
        finally:
            os.remove(expected_path1)
            os.remove(expected_path2)



if __name__ == '__main__':
    


    suite1 = unittest.TestLoader().loadTestsFromTestCase(CacheTest)
    alltests = unittest.TestSuite([suite1])
    
    #selection = unittest.TestSuite()
    #selection.addTest(HouseprintTest('test_get_sensor'))
        
    unittest.TextTestRunner(verbosity=1, failfast=False).run(alltests)



# -*- coding: utf-8 -*-
"""
Created on Thu Jan  7 10:31:00 2016

@author: roel
"""
from opengrid import ureg
import pandas as pd
from dateutil import rrule
import datetime as dt
from itertools import groupby, count
import pytz


def parse_date(d):
    """
    Return a pandas.Timestamp if possible.  
    
    Parameters
    ----------
    d : Datetime, float, int, string or pandas Timestamp
        Anything that can be parsed into a pandas.Timestamp
        
    Returns
    -------
    pts : pandas.Timestamp
    
    Raises
    ------
    ValueError if it was not possible to create a pandas.Timestamp
    """

    if isinstance(d, float) or isinstance(d, int):
        # we have a POSIX timestamp IN SECONDS.
        pts = pd.Timestamp(d, unit='s')
        return pts

    try:
        pts = pd.Timestamp(d)
    except:
        raise ValueError("{} cannot be parsed into a pandas.Timestamp".format(d))
    else:
        return pts


def time_to_timedelta(t):
    """
    Return a pandas Timedelta from a time object

    Parameters
    ----------
    t : datetime.time

    Returns
    -------
    pd.Timedelta

    Notes
    ------
    The timezone of t (if present) is ignored.
    """
    return pd.Timedelta(seconds=t.hour * 3600 + t.minute * 60 + t.second + t.microsecond * 1e-3)


def split_by_day(df, starttime=dt.time.min, endtime=dt.time.max):
    """
    Return a list with dataframes, one for each day

    Parameters
    ----------
    df : pandas DataFrame with datetimeindex
    starttime, endtime :datetime.time objects
        For each day, only return a dataframe between starttime and endtime
        If None, use begin of day/end of day respectively

    Returns
    -------
    list, one dataframe per day.
    """
    if df.empty:
        return None

    df = df[(df.index.time >= starttime) & (df.index.time < endtime)]  # slice between starttime and endtime
    list_df = [group[1] for group in df.groupby(df.index.date)]  # group by date and create list
    return list_df


def unit_conversion_factor(source, target):
    """
    Shorthand function to get a conversion factor for unit conversion.

    Parameters
    ----------
    source, target : str
        Unit as string, should be parsable by pint

    Returns
    -------
    cf : float
        Conversion factor. Multiply the source value with this factor to
        get the target value.  Works only for factorial conversion!

    """

    if not source or not target:
        return 1
    if source == target:
        return 1
    else:
        return 1 * ureg(source).to(target).magnitude


def dayset(start, end):
    """
        Takes a start and end date and returns a set containing all dates between start and end

        Parameters
        ----------
        start : datetime-like object
        end : datetime-like object

        Returns
        -------
        set of datetime objects
    """

    res = []
    for day in rrule.rrule(rrule.DAILY, dtstart=start, until=end):
        res.append(day.date())
    return sorted(set(res))


def split_irregular_date_list(date_list):
    """
    Takes a list of dates and groups it into blocks of continuous dates.
    It returns the begin and end of those blocks

    eg. A list with continuous dates from januari to march and september to october will be split into two lists

    Parameters
    ----------
    date_list : list of datetime.date

    Returns
    -------
    list of tuples of datetime.date
    """
    date_list = sorted(date_list)
    def as_range(g):
        l = list(g)
        return l[0], l[-1]

    return [as_range(g) for _, g in groupby(date_list, key=lambda n, c=count(): n - dt.timedelta(days=next(c)))]


def calculate_temperature_equivalent(temperatures):
    """
    Calculates the temperature equivalent from a series of average daily temperatures
    according to the formula: 0.6 * tempDay0 + 0.3 * tempDay-1 + 0.1 * tempDay-2

    Parameters
    ----------
    series : Pandas Series

    Returns
    -------
    Pandas Series
    """

    ret = 0.6*temperatures + 0.3*temperatures.shift(1) + 0.1*temperatures.shift(2)
    ret.name = 'temp_equivalent'
    return ret


def calculate_degree_days(temperature_equivalent, base_temperature, cooling=False):
    """
    Calculates degree days, starting with a series of temperature equivalent values

    Parameters
    ----------
    temperature_equivalent : Pandas Series
    base_temperature : float
    cooling : bool
        Set True if you want cooling degree days instead of heating degree days

    Returns
    -------
    Pandas Series
    """

    if cooling:
        ret = temperature_equivalent - base_temperature
    else:
        ret = base_temperature - temperature_equivalent

    # degree days cannot be negative
    ret[ret < 0] = 0

    prefix = 'cooling' if cooling else 'heating'
    ret.name = '{}_degree_days_{}'.format(prefix, base_temperature)

    return ret


def last_midnight(timezone):
    """
    Return the timestamp of the last midnight in a given timezone

    Parameters
    ----------
    timezone: str
        pytz timezone

    Returns
    -------
    datetime
    """

    tz = pytz.timezone(timezone)
    now = dt.datetime.now(tz=tz)
    midnight = now.replace(hour=0, minute=0, second=0, microsecond=0)

    return midnight



# coding: utf-8

# In[1]:

# opengrid imports
from opengrid.library import misc, houseprint, caching
from opengrid.library.analysis import DailyAgg
from opengrid import config
c=config.Config()

# other imports
import pandas as pd
import charts
import numpy as np
import os
import datetime as dt
import pytz
BXL = pytz.timezone('Europe/Brussels')


# configuration for the plots
DEV = c.get('env', 'type') == 'dev' # DEV is True if we are in development environment, False if on the droplet
print("Environment configured for development: {}".format(DEV))
if not DEV:
    # production environment: don't try to display plots
    import matplotlib
    matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.dates import MinuteLocator, HourLocator, DateFormatter, AutoDateLocator, num2date

if DEV:
    if c.get('env', 'plots') == 'inline':
        get_ipython().magic(u'matplotlib inline')
    else:
        get_ipython().magic(u'matplotlib qt')
else:
    pass # don't try to render plots
plt.rcParams['figure.figsize'] = 12,8


# In[2]:

# Load houseprint from cache if possible, otherwise build it from source
try:
    hp_filename = os.path.join(c.get('data', 'folder'), 'hp_anonymous.pkl')
    hp = houseprint.load_houseprint_from_file(hp_filename)
    print("Houseprint loaded from {}".format(hp_filename))
except Exception as e:
    print(e)
    print("Because of this error we try to build the houseprint from source")
    hp = houseprint.Houseprint()
sensors = hp.get_sensors(sensortype='electricity') # sensor objects

# Remove some sensors
exclude = [
            '565de0a7dc64d8370aa321491217b85f' # 3E
          ]
solar = [x.key for x in hp.search_sensors(type='electricity', system='solar')]
exclude += solar

for s in sensors:
    if s.key in exclude:
        sensors.remove(s)

hp.init_tmpo()


# The first time, this will take a very looong time to get all the detailed data for building the cache
# Afterwards, this is quick
starttime = dt.time(0, tzinfo=BXL)
endtime = dt.time(5, tzinfo=BXL)
caching.cache_results(hp=hp, sensors=sensors, resultname='elec_min_night_0-5', AnalysisClass=DailyAgg,  
                      agg='min', starttime=starttime, endtime=endtime)

caching.cache_results(hp=hp, sensors=sensors, resultname='elec_max_night_0-5', AnalysisClass=DailyAgg, 
                      agg='max', starttime=starttime, endtime=endtime)


# In[ ]:

cache_min = caching.Cache(variable='elec_min_night_0-5')
cache_max = caching.Cache(variable='elec_max_night_0-5')
dfdaymin = cache_min.get(sensors=sensors)
dfdaymax = cache_max.get(sensors=sensors)
dfdaymin.info()


# The next plot shows that some periods are missing.  Due to the cumulative nature of the electricity counter, we still have the total consumption.  However, it is spread out of the entire period.  So we don't know the standby power during these days, and we have to remove those days.  

# In[ ]:

if DEV:
    sensor = hp.search_sensors(key='3aa4')[0]
    df = sensor.get_data(head=pd.Timestamp('20151117'), tail=pd.Timestamp('20160104'))
    charts.plot(df, stock=True, show='inline')


# In[ ]:

def is_submeter(sensor, dfdaymin, dfdaymax):
    """
    Return True if this sensor is a sub-meter
    
    sensor = sensor object
    """
    
    
    other_sensors = sensor.device.get_sensors(sensortype='electricity')
    other_sensors.remove(sensor)
    if len(other_sensors) == 0:
        print("\n{} - {}: no other sensors, this must be main.".format(sensor.device.key, sensor.description))
        return False
    else:
        print("\n{} - {}: comparing with:".format(sensor.device.key, sensor.description))
        for o in other_sensors:
            # we only check the values for last day
            print("* {}:".format(o.description))
            sensormin = dfdaymin.ix[-1,sensor.key]
            sensormax = dfdaymax.ix[-1,sensor.key]
            try:
                othermin = dfdaymin.ix[-1].dropna()[o.key]
                othermax = dfdaymax.ix[-1].dropna()[o.key]
            except:
                print("  No data found for sensor {}".format(o.description))
                pass
            else:
                if (sensormin <= othermin) and (sensormax <= othermax):
                    print("  {} has lower daily min AND max, so it is a submeter.".format(sensor.description))                   
                    return True
                else:
                    print("  {} has higher daily min and/or max, we look further.".format(sensor.description))
        else:
            print("All other sensors have no data OR lower daily min and max. {} must be main.".format(sensor.description))
            return False
    
    


# In[ ]:

# The function is_submeter makes one obvious error: see results for FL03001566
for col in dfdaymin:
    is_submeter(hp.find_sensor(col), dfdaymin, dfdaymax)


# In[ ]:

# Clean out the data: 
# First remove sensors that are submeters
for col in dfdaymin:
    if is_submeter(hp.find_sensor(col), dfdaymin, dfdaymax):
        print("\n!!Removing submeter {}".format(col))
        dfdaymin = dfdaymin.drop(col, axis=1)
# Now remove days with too low values to be realistic
dfdaymin[dfdaymin < 10] = np.nan
# Now remove days where the minimum=maximum (within 1 Watt difference)
dfdaymin[(dfdaymax - dfdaymin) < 1] = np.nan


# In[ ]:

dfdaymin.info()


# In[ ]:

if DEV:
    charts.plot(dfdaymin, stock=True, show='inline')


# In[ ]:

standby_statistics = dfdaymin.T.describe(percentiles=[0.1,0.5,0.9]).T


# In[ ]:

if DEV:
    charts.plot(standby_statistics[['10%', '50%', '90%']], stock=True, show='inline')


# In[ ]:

# Get detailed profiles for the last day
now = pd.Timestamp('now', tz=BXL)
dt_start_of_last_day = pd.Timestamp(dfdaymin.index[-1].date(), tz=BXL)
dt_end_of_last_day = dt_start_of_last_day + pd.Timedelta(hours=endtime.hour, minutes=endtime.minute)
sensors = map(hp.find_sensor, dfdaymin.columns)
df_details = hp.get_data(sensors = sensors, head=dt_start_of_last_day, tail=dt_end_of_last_day)
df_details.fillna(method='ffill', inplace=True)
df_details.fillna(method='bfill', inplace=True)


# ### Boxplot approach.  Possible for a period of maximum +/- 2 weeks. 

# In[ ]:

# choose a period
look_back_days = 10
dt_start_of_period = dt_start_of_last_day - pd.Timedelta(days=look_back_days-1)
dfdaymin_period = dfdaymin.ix[dt_start_of_period:].dropna(axis=1, how='all')


# In[ ]:

box = [dfdaymin_period.loc[i,:].dropna().values for i in dfdaymin_period.index]
for sensor in dfdaymin_period.columns:
    fig=plt.figure(figsize=(10,5))
    ax1=plt.subplot(121)
    ax1.boxplot(box, positions=range(len(box)), notch=False)
    ax1.plot(range(len(box)), dfdaymin_period[sensor], 'rD', ms=10, label='Standby power')
    xticks = [x.strftime(format='%d/%m') for x in dfdaymin_period.index]
    plt.xticks(range(len(box)), xticks, rotation='vertical')
    plt.title(hp.find_sensor(sensor).device.key + ' - ' + hp.find_sensor(sensor).description)
    ax1.grid()
    ax1.set_ylabel('Watt')
    plt.legend(numpoints=1, frameon=False)
    ax2=plt.subplot(122)
    try:
        ax2.plot_date(df_details[sensor].index, df_details[sensor].values, 'b-', label='Last night')
        ax2.xaxis_date(tz=BXL) #Put timeseries plot in local time
        # rotate the labels
        plt.xticks(rotation='vertical')
        ax2.set_ylabel('Watt')
        ax2.set_xlabel('Local time (BXL)')
        ax2.grid()
        
        xax = ax2.get_xaxis() # get the x-axis
        xax.set_major_locator(HourLocator())
        xax.set_minor_locator(MinuteLocator(30))
        
        adf = xax.get_major_formatter() # the the auto-formatter

        adf.scaled[1./24] = '%H:%M'  # set the < 1d scale to H:M
        adf.scaled[1.0] = '%Y-%m-%d' # set the > 1d < 1m scale to Y-m-d
        adf.scaled[30.] = '%Y-%m' # set the > 1m < 1Y scale to Y-m
        adf.scaled[365.] = '%Y' # set the > 1y scale to Y

        plt.legend(loc='upper right', frameon=False)
        plt.tight_layout()
    except Exception as e:
        print(e)
    else:
        plt.savefig(os.path.join(c.get('data', 'folder'), 'figures', 'standby_horizontal_'+sensor+'.png'), dpi=100)
        pass
    
    if not DEV:
        plt.close()


# ### Percentile approach.  Useful for longer time periods, but tweaking of graph still needed

# In[ ]:

# choose a period
look_back_days = 40
dt_start_of_period = dt_start_of_last_day - pd.Timedelta(days=look_back_days-1)
dfdaymin_period = dfdaymin.ix[dt_start_of_period:].dropna(axis=1, how='all')
df = dfdaymin_period.join(standby_statistics[['10%', '50%', '90%']], how='left')    


# In[ ]:

for sensor in dfdaymin_period.columns:
    plt.figure(figsize=(10,8))
    ax1=plt.subplot(211)
    ax1.plot_date(df.index, df[u'10%'], '-', lw=2, color='g', label=u'10% percentile')
    ax1.plot_date(df.index, df[u'50%'], '-', lw=2, color='orange', label=u'50% percentile')
    ax1.plot_date(df.index, df[u'90%'], '-', lw=2, color='r', label=u'90% percentile')
    ax1.plot_date(df.index, df[sensor], 'rD', ms=7, label='Your standby power') 
    ax1.legend()
    locs, lables=plt.xticks()
    xticks = [x.strftime(format='%d/%m') for x in num2date(locs)]
    plt.xticks(locs, xticks, rotation='vertical')
    plt.title(hp.find_sensor(sensor).device.key + ' - ' + sensor)
    ax1.grid()
    ax1.set_ylabel('Watt')
    
    ax2=plt.subplot(212)
    try:
        ax2.plot_date(df_details[sensor].index, df_details[sensor].values, 'b-', label='Detailed consumption of last night')
        ax2.xaxis_date(tz=BXL) #Put timeseries plot in local time
        # rotate the labels
        plt.xticks(rotation='vertical')
        ax2.set_ylabel('Watt')
        ax2.set_xlabel('Local time (BXL)')
        ax2.grid()
        
        xax = ax2.get_xaxis() # get the x-axis
        xax.set_major_locator(HourLocator())
        xax.set_minor_locator(MinuteLocator(30))
        
        adf = xax.get_major_formatter() # the the auto-formatter

        adf.scaled[1./24] = '%H:%M'  # set the < 1d scale to H:M
        adf.scaled[1.0] = '%Y-%m-%d' # set the > 1d < 1m scale to Y-m-d
        adf.scaled[30.] = '%Y-%m' # set the > 1m < 1Y scale to Y-m
        adf.scaled[365.] = '%Y' # set the > 1y scale to Y
        
        plt.legend(loc='upper right', frameon=False)
        plt.tight_layout()
    except Exception as e:
        print(e)
    else:
        plt.savefig(os.path.join(c.get('data', 'folder'), 'figures', 'standby_vertical_'+sensor+'.png'), dpi=100)
        pass
    if not DEV:
        plt.close()


# In[ ]:






# -*- coding: utf-8 -*-
"""
<author>
<Date>

Functions to fetch (historic, current or prognosed) weather data from wunderground weather stations.
Required: API key, to be added to opengrid.cfg.

"""

# import libraries specific to wunderground script
import datetime
import pytz
import pandas as pd
import pdb


import urllib2 
import json 
from pprint import pprint
import time


class Wunderground(object):
    """
    Class to manage api calls to fetch current and historic weather data from
    Wunderground weather stations
    
    This class avoids calling the api again for getting the current conditions
    if they have been obtained within time_delay_min. This is handled by the
    following attributes:
    
    * timestamp_current: timestamp when last call to fetch current conditions
      was succesfully launched
    * json_current : json object returned with the last call
    * time_delay : if timestamp_current is within this time_delay from the
      actual time, use the existing json_current.  Else, call Wunderground to 
      update json_current, and update timestamp_current accordingly.
    """

    def __init__(self, apikey, city, time_delay=20*60):
        """
        Parameters
        ----------
        * apikey : String
            Wunderground API key (can be obtained after subscription)
        * city : String
            Location of weather station
        * time_delay: float
            Time delay in seconds for calling api since last call for current conditions
        """
        
        self.apikey = apikey
        self.city = city
        self.time_delay = time_delay
        
        self.json_current = None
        self.timestamp_current = 0
        
    def reset(self):
        """
        Reset json_current and timestam_current
        """
        
        self.json_current = None
        self.timestamp_current = 0
        
    
    def _fetch_current(self):
        """
        Call Wunderground to get current conditions and overwrite
        json_current and timestamp_current
        """
        URL = ''.join(['http://api.wunderground.com/api/',self.apikey,'/geolookup/conditions/q/EU/',self.city,'.json'])
        f = urllib2.urlopen(URL) 
        json_string = f.read() 
        self.json_current = json.loads(json_string)
        self.timestamp_current = time.time()
        
        print("Obtained current conditions and saved json in self.json_current")
        
    def fetch_current(self):
        """
        Call Wunderground to get current conditions IF NEEDED based on current
        time, self.timestamp_current and self.time_delay.  
        If Wunderground is to be called, overwrite json_current and timestamp_current
        """        
        
        if time.time() < (self.timestamp_current + self.time_delay):
            print("No need to call Wunderground, using cached current conditions.")
        else:
            self._fetch_current()
            
    
    def show_properties(self):
        """
        Print all properties that can be obtained from the current conditions
        todo!!
        """
        
        try:
            print("These are the known current properties:\n")
            for prop in self.json_current['current_observation'].keys():
                print(prop)
        except:
            print("Problem reading self.json_current")
            

    def get_current(self, prop):
        """
        Extract current value for prop from self.json_current. 
        Fetch current conditions from Wunderground if needed, depending on 
        current time, self.timestamp_current and self.time_delay. 
        
        Parameters
        ----------
        * prop : String
            Type of weather property to look up.
            
        Returns
        -------
        value, timestamp : value and associated timestamp
        
        Note
        ----
        If prop cannot be found, print all possible properties and raise a 
        ValueError
        """
            
        self.fetch_current()
                
        try:
            curr_value = float(self.json_current['current_observation'][prop] )
        except:
            print("Property not found.")            
            self.show_properties()
            raise ValueError('Property {} not found.  See show_properties() to get a list with all possibilities'.format(prop))
        
    
        return curr_value, self.timestamp_current
        

#Script to obtain CURRENT weather detail readings
def fetch_curr_conditions(apikey, city, prop='temp_c'):
    '''
    Generic function for getting current weather conditions in a specified city.
        
    Parameters
    ----------
    * apikey : String
        Wunderground API key (can be obtained after subscription)
    * city : String
        location of weather station
    * prop : String
        Type of weather property to look up.  
        
    Returns
    -------
    
    
    Author: Ryton & comments from Saroele

    Output:  double return with value, date/time. If an error occurs, the json response will be printed.
    '''
    #
    
    URL = ''.join(['http://api.wunderground.com/api/',apikey,'/geolookup/conditions/q/EU/',city,'.json'])
    f = urllib2.urlopen(URL) 
    json_string = f.read() 
    parsed_json = json.loads(json_string) 
    #pprint(parsed_json)
    
    try:
        location = parsed_json['location']['city'] 
        curr_value = float(parsed_json['current_observation'][prop] )
        currdate = datetime.date.today()
    except:
        pprint(parsed_json)
        raise
    
    f.close()

    return curr_value, currdate

def details_xdaysago(key,city,x_days_ago = 5,prop = "temp_c",columnname ='T_out'):
    """
    Get more subhourly details for a single day, x days ago
    
    todo: complete docstring
    """
    x_days_ago_date =  datetime.datetime.today()- datetime.timedelta(days=x_days_ago)   
    temp_values = fetch_historic_temp_bydate(key,city,x_days_ago_date,prop,columnname)
    return temp_values


#some scripts for HISTORIC data fetching

def fetch_historic_tempYMD(key,city,year,month,day, prop = 'tempm',columnname= 'T_out' ):
    """
    Get temp df using year month day.

    
    Example URL: http://api.wunderground.com/api/<key>/history_20140808/q/EU/Leuven.json
    
    todo: complete docstring    
    """    
    d = datetime.datetime(year,month,day,0,0)
    datestr = '{:%Y%m%d}'.format(d)
    
    URL = ''.join(['http://api.wunderground.com/api/',key,'/history_',datestr,'/q/BE/',city,'.json'])
    f = urllib2.urlopen(URL) 
    json_string = f.read() 
    parsed_json = json.loads(json_string) 
    #print json_string
    #pprint(parsed_json["history"]['dailysummary'])
    hr = "hour"
    minu = "min"
    time_list=[]
    temp_c_list= []

    for  entry in parsed_json["history"]["observations"]:
        #pprint(entry)
        hour_value = entry["date"][hr]
        min_value = entry["date"][minu]
        temp_c = entry[prop]
        temp_c_list.append(float(temp_c))
        concattime = datetime.datetime(year,month, day,int(hour_value),int(min_value))
        time_list.append(concattime)
        
        
        #print concattime, temp_c
    # print shape(temp_c_list), shape(time_list)
    Tout_h = pd.DataFrame(temp_c_list,columns = [columnname],index = time_list)
    return Tout_h
    f.close()
    
def fetch_historic_temp_bydate(key,city,date_object, prop = 'tempm',columnname= 'T_out' ):
#same as above, but with dateobject as input. hour / min are ignored
    year = date_object.year
    month = date_object.month
    day = date_object.day
    Tout_h = fetch_historic_tempYMD(key,city,year,month,day, prop ,columnname)
    return Tout_h

#scripts for Historic day-AVERAGE weather fetching

def fetch_historic_dayaverage(key,city,year,month,day,prop = "meantempm",columnname ='T_out'):
    # example URL: http://api.wunderground.com/api/<key>/history_20140808/q/EU/Leuven.json
    # get temp df using year month day.
    #city = 'Geel'   
    d = datetime.datetime(year,month,day,0,0)
    datestr = '{:%Y%m%d}'.format(d)
    
    URL = ''.join(['http://api.wunderground.com/api/',key,'/history_',datestr,'/q/BE/',city,'.json'])
    f = urllib2.urlopen(URL) 
    json_string = f.read() 
    parsed_json = json.loads(json_string) 
    
    #print json_string
    #pprint(parsed_json["history"]['dailysummary'])
    hr = "hour"
    minu = "min"
    
    time_list=[]
    temp_c_list= []
    
    for  entry in parsed_json["history"]['dailysummary']:
        #"pprint(entry)
        hour_value = entry["date"][hr]
        min_value = entry["date"][minu]
        temp_c = entry[prop]
        temp_c_list.append(temp_c)
        concattime = datetime.datetime(year,month, day,int(hour_value),int(min_value))
        time_list.append(concattime)
        #print concattime, temp_c
    #Tout_h = pd.DataFrame([temp_c_list],columns = ['T_out'],index = time_list)
    #parse_dates=['Date'], dayfirst=True, index_col='Date'
    Tout_h = pd.DataFrame([temp_c_list],columns = [columnname], index=time_list)
    #
    
    #Tout_h_T.set_index(time_list)
    f.close()
    return Tout_h
    
    
def fetch_historic_dayaverage_by_date(key,city,date_object,prop = 'meantempm',columnname ='T_out'):
#same as above, but with dateobject as input. hour / min are ignored
    year = date_object.year
    month = date_object.month
    day = date_object.day
    Tout_h = fetch_historic_dayaverage(key,city,year,month,day,prop,columnname);
    return Tout_h

def average_temp_xdaysago(key,city,x_days_ago = 5,prop = 'meantempm',columnname ='T_out'):
#get more subhourly details for a single day, eg. day 5
    x_days_ago_date =  datetime.datetime.today()- datetime.timedelta(days=x_days_ago)   
    temp_values = fetch_historic_dayaverage_by_date(key,city,x_days_ago_date,prop,columnname)
    return temp_values


	



import requests
import bs4
import datetime as dt
import pandas as pd
from .misc import calculate_temperature_equivalent, calculate_degree_days


def get_kmi_current_month(include_temperature_equivalent=True, include_heating_degree_days=True,
                          heating_base_temperatures=[16.5], include_cooling_degree_days=True,
                          cooling_base_temperatures=[18], solar_duration_as_minutes=False,
                          include_wind_power=False):
    """
    Gets the current month table from http://www.meteo.be/meteo/view/nl/123763-Huidige+maand.html
    and parse it into a Pandas DataFrame

    Parameters
    ----------
    include_temperature_equivalent : bool
    include_heating_degree_days : bool
    heating_base_temperatures : list of floats
    include_cooling_degree_days : bool
    cooling_base_temperatures : list of floats
    solar_duration_as_minutes : bool
    include_wind_power : bool

    Returns
    -------
    Pandas DataFrame
    """
    # start out by getting the html from the website
    html = fetch_website()
    df = parse(html=html, solar_duration_as_minutes=solar_duration_as_minutes)

    if include_temperature_equivalent or include_heating_degree_days or include_cooling_degree_days:
        temp_equiv = calculate_temperature_equivalent(temperatures=df.temp_gem)
    if include_temperature_equivalent:
        df = df.join(temp_equiv)

    if include_heating_degree_days:
        for base_temperature in heating_base_temperatures:
            degree_days = calculate_degree_days(temperature_equivalent=temp_equiv, base_temperature=base_temperature)
            df = df.join(degree_days)
    if include_cooling_degree_days:
        for base_temperature in cooling_base_temperatures:
            degree_days = calculate_degree_days(temperature_equivalent=temp_equiv, base_temperature=base_temperature,
                                                cooling=True)
            df = df.join(degree_days)
    if include_wind_power:
        df['wind_power'] = df.wind_snelh ** 3

    return df


def fetch_website(url="http://www.meteo.be/meteo/view/nl/123763-Huidige+maand.html"):
    """
    Fetch the website containing the data from http://www.meteo.be/meteo/view/nl/123763-Huidige+maand.html

    Returns
    -------
    str
    """

    r = requests.get(url)

    if r.status_code == 200:
        return r.text
    else:
        raise Exception("Seems like you got code {}".format(r.status_code))


def parse(html, solar_duration_as_minutes=False):
    """
    Parse the html from http://www.meteo.be/meteo/view/nl/123763-Huidige+maand.html to a Pandas DataFrame

    Parameters
    ----------
    html : str
    solar_duration_as_minutes : bool

    Returns
    -------
    Pandas DataFrame
    """
    soup = bs4.BeautifulSoup(html, "html.parser")
    day_values = soup.findAll("tbody")[1]  # the table we want is the second table called 'tbody'
    table_rows = day_values.findAll("tr")

    # parse column names. Take the titles, but remove periods, linebreaks and other artefacts
    titles = table_rows[0].findAll("th")
    column_names = ["_".join(title.text.replace(".", "").split()).lower() for title in titles]

    # parse rows
    # The table has a stupid date format, so it is quite hard to infer the correct date.
    # We start from the back, because the last row should be from yesterday
    # We then work our way back from the date we know into the past
    rows = []
    last_date = dt.date.today()
    for row in reversed(table_rows[2:]):
        values = []
        for title, td in zip(column_names, row.findAll("td")):
            if title == 'datum':
                day = int(td.text.split(" ")[0])  # get day as int from text
                while day != last_date.day:
                    last_date = last_date - dt.timedelta(days=1)
                values.append(last_date)
            elif title == 'zon_duur':
                try:
                    hour, minute = td.text.split(":")
                    if solar_duration_as_minutes:
                        time = int(hour)*60 + int(minute)
                    else:
                        time = dt.time(hour=int(hour), minute=int(minute))
                except ValueError:
                    if solar_duration_as_minutes:
                        time = float('NaN')
                    else:
                        time = pd.NaT
                values.append(time)
            else:
                try:
                    val = float(td.text.replace(",", "."))
                except ValueError:
                    val = float('NaN')
                values.append(val)
        rows.append(values)

    # create DataFrame
    df = pd.DataFrame(rows, columns=column_names).set_index('datum')
    df.index = pd.DatetimeIndex(df.index)
    df = df.sort_index().tz_localize('Europe/Brussels')

    return df



# -*- coding: utf-8 -*-
"""
Created on Mon Dec 30 02:37:25 2013

@author: roel
"""

import os, sys
import unittest
import inspect
import numpy as np
import pdb
import datetime as dt
import pandas as pd
import pytz

test_dir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
# add the path to opengrid to sys.path
sys.path.append(os.path.join(test_dir, os.pardir, os.pardir))
from opengrid.library import fluksoapi

class FluksoapiTest(unittest.TestCase):
    """
    Class for testing the module fluksoapi
    """

    def test_consolidate_single(self):
        """Return abspath if a single file found"""
        
        datafolder = os.path.join(test_dir, 'data')
        self.assertRaises(ValueError, fluksoapi.consolidate_sensor, datafolder, 'f81fb35a62f59a987d8eea3ffc845ed0')
        
        csv_expected = os.path.join(datafolder, 'FL12345678_sensorS_FROM_2014-01-07_16-02-00_TO_2014-01-08_16-01-00.csv' )
        self.assertEqual(csv_expected,
                         fluksoapi.consolidate_sensor(datafolder, 'sensorS'))
                         
        
                         
    def test_consolidate_multiple(self):
        """Consolidate and return single filename if more than one file found"""
        
        datafolder = os.path.join(test_dir, 'data')
        csv_expected = os.path.join(datafolder, 'FL12345678_sensorD_FROM_2014-01-07_08-02-00_TO_2014-01-08_16-01-00.csv' )
        self.assertEqual(csv_expected, fluksoapi.consolidate_sensor(datafolder, 'sensorD'))
                         
        os.remove(csv_expected)


    def test_consolidate_raises(self):
        """Raise ValueError if no file found"""
        
        datafolder = os.path.join(test_dir, 'data')
        self.assertRaises(ValueError, fluksoapi.consolidate_sensor, datafolder, 'thissensordoesnotexist')
        

        
    def test_consolidate(self):
        """Consolidating 2 files and checking variable"""
        
        datafolder = os.path.join(test_dir, 'data')        
        new_csv=fluksoapi.consolidate_sensor(folder = datafolder, 
                                             sensor = 'sensorD')
              
        ts1 = fluksoapi.load_file(os.path.join(datafolder, 'FL12345678_sensorD_FROM_2014-01-07_08-02-00_TO_2014-01-08_08-01-00.csv'))
        self.assertTrue(np.isnan(ts1['sensorD'].loc[dt.datetime(2014,1,8,8,0,0, tzinfo=pytz.UTC)]))       
        ts2 = fluksoapi.load_file(os.path.join(datafolder, 'FL12345678_sensorD_FROM_2014-01-07_16-02-00_TO_2014-01-08_16-01-00.csv'))    
        #ts = fluksoapi.load_file(os.path.join(datafolder, 'f81fb35a62f59a987d8eea3ffc845ed0_FROM_2014-01-07_08-02-00_TO_2014-01-08_16-01-00.csv'))
        #pdb.set_trace()        

        ts = fluksoapi.load_file(new_csv)        
        self.assertEqual(ts.index[0], ts1.index[0])
        self.assertEqual(ts.index[-1], ts2.index[-1])
        self.assertEqual(ts['sensorD'].loc['2014-01-08 08:00:00'], 1120.0, "Last file should overwrite identical indices")
        os.remove(new_csv)


    def test_consolidate_with_hidden_file(self):
        """Consolidate should skip hidden file"""
        
        datafolder = os.path.join(test_dir, 'data')        
        new_csv=fluksoapi.consolidate_sensor(folder = datafolder, 
                                             sensor = 'sensorH')
                                             
        self.assertEqual(new_csv, os.path.join(datafolder, 'FL12345678_sensorH_FROM_2014-01-07_12-02-00_TO_2014-01-08_16-01-00.csv'))
        os.remove(new_csv)
        
        
    def test_consolidate_single_file(self):
        """Consolidating a single file should NOT consolidate but should return the file"""
        
        datafolder = os.path.join(test_dir, 'data')        
        new_csv=fluksoapi.consolidate_sensor(folder = datafolder, 
                                             sensor = 'sensorS')
              
        self.assertEqual(new_csv, os.path.join(datafolder,'FL12345678_sensorS_FROM_2014-01-07_16-02-00_TO_2014-01-08_16-01-00.csv'))
        
    def test_consolidate_day(self):
        """Consolidating 2 files for a single day and checking variable"""
        
        datafolder = os.path.join(test_dir, 'data')        
        new_csv=fluksoapi.consolidate_sensor(folder = datafolder, 
                                             sensor = 'sensorD',
                                             dt_day = dt.datetime(2014,1,7))
              
        ts1 = fluksoapi.load_file(os.path.join(datafolder, 'FL12345678_sensorD_FROM_2014-01-07_08-02-00_TO_2014-01-08_08-01-00.csv'))
        self.assertTrue(np.isnan(ts1['sensorD'].loc[dt.datetime(2014,1,8,8,0,0, tzinfo=pytz.UTC)]))       
        ts2 = fluksoapi.load_file(os.path.join(datafolder, 'FL12345678_sensorD_FROM_2014-01-07_16-02-00_TO_2014-01-08_16-01-00.csv'))

        ts = fluksoapi.load_file(new_csv)
        self.assertEqual(ts.index[0], ts1.index[0])
        self.assertEqual(ts.index[-1], dt.datetime(2014,1,8,0,0,0, tzinfo=pytz.UTC))
        
        os.remove(new_csv)
        
    def test_load_file(self):
        """load_file should return a pandas dataframe with localized index (UTC)"""
        
        datafolder = os.path.join(test_dir, 'data')          
        df = fluksoapi.load_file(os.path.join(datafolder, 'FL12345678_sensorD_FROM_2014-01-07_08-02-00_TO_2014-01-08_08-01-00.csv'))
        self.assertIsInstance(df, pd.DataFrame)
        self.assertEqual(df.index.tz, pytz.UTC, "the tz is {} instead of UTC".format(df.index.tz))
        self.assertListEqual(df.columns.tolist(), ['sensorD'])


    def test_parse_date_from_datetime(self):
        """Parsing a datetime into a pandas.Timestamp"""
        
        BXL = pytz.timezone('Europe/Brussels')
        dt_ = BXL.localize(dt.datetime(2014,11,23,1,2,3))
        epoch = pytz.UTC.localize(dt.datetime(1970,1,1,0,0,0))
        epoch_expected = (dt_ - epoch).total_seconds()
        
        pts = fluksoapi._parse_date(dt_)
        self.assertEqual(pts.value/1e9, epoch_expected)
        
    
    def test_parse_date_from_datetime_naive(self):
        """Parsing a naïve datetime into a pandas.Timestamp makes it UTC"""
        
        dt_ = pytz.UTC.localize(dt.datetime(2014,11,23,1,2,3))
        epoch = pytz.UTC.localize(dt.datetime(1970,1,1,0,0,0))
        epoch_expected = (dt_ - epoch).total_seconds()
        
        pts = fluksoapi._parse_date(dt.datetime(2014,11,23,1,2,3))
        self.assertEqual(pts.value/1e9, epoch_expected)
        
        
    def test_parse_date_from_posix(self):
        """Parsing a float"""
        
        pts = fluksoapi._parse_date(1416778251.460574)
        self.assertEqual(1416778251.460574, pts.value/1e9)
        
    def test_parse_date_from_string(self):
        """Parsing some commong types of strings"""
        
        dt_ = pytz.UTC.localize(dt.datetime(2014,11,23,1,2,3))
        epoch = pytz.UTC.localize(dt.datetime(1970,1,1,0,0,0))
        epoch_expected = (dt_ - epoch).total_seconds()
        
        pts = fluksoapi._parse_date('20141123 01:02:03')
        self.assertEqual(pts.value/1e9, epoch_expected)   
        
        pts = fluksoapi._parse_date('2014-11-23 01:02:03')
        self.assertEqual(pts.value/1e9, epoch_expected)   

        pts = fluksoapi._parse_date('2014-11-23T010203')
        self.assertEqual(pts.value/1e9, epoch_expected)   


if __name__ == '__main__':
    
    #http://stackoverflow.com/questions/4005695/changing-order-of-unit-tests-in-python    
    ln = lambda f: getattr(FluksoapiTest, f).im_func.func_code.co_firstlineno
    lncmp = lambda _, a, b: cmp(ln(a), ln(b))
    unittest.TestLoader.sortTestMethodsUsing = lncmp

    suite1 = unittest.TestLoader().loadTestsFromTestCase(FluksoapiTest)
    alltests = unittest.TestSuite([suite1])
    
    #selection = unittest.TestSuite()
    #selection.addTest(HouseprintTest('test_get_sensor'))
        
    unittest.TextTestRunner(verbosity=1, failfast=False).run(alltests)
